@@ -165,7 +165,7 @@ Protocol Key::protocol() const
 
 const char *Key::protocolAsString() const
 {
-    return key ? gpgme_get_protocol_name(key->protocol) : 0 ;
+    return key ? gpgme_get_protocol_name(key->protocol) : nullptr ;
 }
 
 bool Key::isRevoked() const
@@ -252,26 +252,26 @@ bool Key::isDeVs() const
 
 const char *Key::issuerSerial() const
 {
-    return key ? key->issuer_serial : 0 ;
+    return key ? key->issuer_serial : nullptr ;
 }
 const char *Key::issuerName() const
 {
-    return key ? key->issuer_name : 0 ;
+    return key ? key->issuer_name : nullptr ;
 }
 const char *Key::chainID() const
 {
-    return key ? key->chain_id : 0 ;
+    return key ? key->chain_id : nullptr ;
 }
 
 const char *Key::keyID() const
 {
-    return key && key->subkeys ? key->subkeys->keyid : 0 ;
+    return key && key->subkeys ? key->subkeys->keyid : nullptr ;
 }
 
 const char *Key::shortKeyID() const
 {
     if (!key || !key->subkeys || !key->subkeys->keyid) {
-        return 0;
+        return nullptr;
     }
     const int len = strlen(key->subkeys->keyid);
     if (len > 8) {
@@ -299,7 +299,7 @@ const char *Key::primaryFingerprint() const
 
 unsigned int Key::keyListMode() const
 {
-    return key ? convert_from_gpgme_keylist_mode_t(key->keylist_mode) : 0 ;
+    return key ? convert_from_gpgme_keylist_mode_t(key->keylist_mode) : 0;
 }
 
 const Key &Key::mergeWith(const Key &other)
@@ -358,14 +358,14 @@ void Key::update()
                         KeyListMode::Signatures |
                         KeyListMode::SignatureNotations |
                         KeyListMode::Validate |
-                        KeyListMode::WithTofu);
+                        KeyListMode::WithTofu |
+                        KeyListMode::WithKeygrip);
     Error err;
     auto newKey = ctx->key(primaryFingerprint(), err, true);
     // Not secret so we get the information from the pubring.
-    if (newKey.isNull())
-      {
+    if (newKey.isNull()) {
         newKey = ctx->key(primaryFingerprint(), err, false);
-      }
+    }
     delete ctx;
     if (err) {
         return;
@@ -410,7 +410,7 @@ gpgme_sub_key_t find_subkey(const shared_gpgme_key_t &key, unsigned int idx)
             }
         }
     }
-    return 0;
+    return nullptr;
 }
 
 gpgme_sub_key_t verify_subkey(const shared_gpgme_key_t &key, gpgme_sub_key_t subkey)
@@ -422,10 +422,10 @@ gpgme_sub_key_t verify_subkey(const shared_gpgme_key_t &key, gpgme_sub_key_t sub
             }
         }
     }
-    return 0;
+    return nullptr;
 }
 
-Subkey::Subkey() : key(), subkey(0) {}
+Subkey::Subkey() : key(), subkey(nullptr) {}
 
 Subkey::Subkey(const shared_gpgme_key_t &k, unsigned int idx)
     : key(k), subkey(find_subkey(k, idx))
@@ -446,12 +446,12 @@ Key Subkey::parent() const
 
 const char *Subkey::keyID() const
 {
-    return subkey ? subkey->keyid : 0 ;
+    return subkey ? subkey->keyid : nullptr ;
 }
 
 const char *Subkey::fingerprint() const
 {
-    return subkey ? subkey->fpr : 0 ;
+    return subkey ? subkey->fpr : nullptr ;
 }
 
 Subkey::PubkeyAlgo Subkey::publicKeyAlgorithm() const
@@ -589,7 +589,7 @@ gpgme_user_id_t find_uid(const shared_gpgme_key_t &key, unsigned int idx)
             }
         }
     }
-    return 0;
+    return nullptr;
 }
 
 gpgme_user_id_t verify_uid(const shared_gpgme_key_t &key, gpgme_user_id_t uid)
@@ -601,10 +601,10 @@ gpgme_user_id_t verify_uid(const shared_gpgme_key_t &key, gpgme_user_id_t uid)
             }
         }
     }
-    return 0;
+    return nullptr;
 }
 
-UserID::UserID() : key(), uid(0) {}
+UserID::UserID() : key(), uid(nullptr) {}
 
 UserID::UserID(const shared_gpgme_key_t &k, gpgme_user_id_t u)
     : key(k), uid(verify_uid(k, u))
@@ -656,22 +656,27 @@ std::vector<UserID::Signature> UserID::signatures() const
 
 const char *UserID::id() const
 {
-    return uid ? uid->uid : 0 ;
+    return uid ? uid->uid : nullptr ;
 }
 
 const char *UserID::name() const
 {
-    return uid ? uid->name : 0 ;
+    return uid ? uid->name : nullptr ;
 }
 
 const char *UserID::email() const
 {
-    return uid ? uid->email : 0 ;
+    return uid ? uid->email : nullptr ;
 }
 
 const char *UserID::comment() const
 {
-    return uid ? uid->comment : 0 ;
+    return uid ? uid->comment : nullptr ;
+}
+
+const char *UserID::uidhash() const
+{
+    return uid ? uid->uidhash : nullptr ;
 }
 
 UserID::Validity UserID::validity() const
@@ -724,6 +729,76 @@ TofuInfo UserID::tofuInfo() const
     return TofuInfo(uid->tofu);
 }
 
+static gpgme_key_sig_t find_last_valid_sig_for_keyid (gpgme_user_id_t uid,
+                                                      const char *keyid)
+{
+    if (!keyid) {
+        return nullptr;
+    }
+    gpgme_key_sig_t ret = NULL;
+    for (gpgme_key_sig_t s = uid->signatures ; s ; s = s->next) {
+        if (s->keyid && !strcmp(keyid, s->keyid)) {
+            if (!s->expired && !s->revoked && !s->invalid && !s->status) {
+                if (!ret) {
+                    ret = s;
+                } else if (ret && ret->timestamp <= s->timestamp) {
+                    /* Equals because when the timestamps are the same we prefer
+                       the last in the list */
+                    ret = s;
+                }
+            }
+        }
+    }
+    return ret;
+}
+
+const char *UserID::remark(const Key &remarker, Error &err) const
+{
+    if (!uid || remarker.isNull()) {
+        err = Error::fromCode(GPG_ERR_GENERAL);
+        return nullptr;
+    }
+
+    if (key->protocol != GPGME_PROTOCOL_OpenPGP) {
+        return nullptr;
+    }
+
+    if (!(key->keylist_mode & GPGME_KEYLIST_MODE_SIG_NOTATIONS) ||
+        !(key->keylist_mode & GPGME_KEYLIST_MODE_SIGS)) {
+        err = Error::fromCode(GPG_ERR_NO_DATA);
+        return nullptr;
+    }
+
+    gpgme_key_sig_t s = find_last_valid_sig_for_keyid(uid, remarker.keyID());
+
+    if (!s) {
+        return nullptr;
+    }
+
+    for (gpgme_sig_notation_t n = s->notations; n ; n = n->next) {
+        if (n->name && !strcmp(n->name, "rem@gnupg.org")) {
+            return n->value;
+        }
+    }
+    return nullptr;
+}
+
+std::vector<std::string> UserID::remarks(std::vector<Key> keys, Error &err) const
+{
+    std::vector<std::string> ret;
+
+    for (const auto &key: keys) {
+        const char *rem = remark(key, err);
+        if (err) {
+            return ret;
+        }
+        if (rem) {
+            ret.push_back(rem);
+        }
+    }
+    return ret;
+}
+
 //
 //
 // class Signature
@@ -739,7 +814,7 @@ gpgme_key_sig_t find_signature(gpgme_user_id_t uid, unsigned int idx)
             }
         }
     }
-    return 0;
+    return nullptr;
 }
 
 gpgme_key_sig_t verify_signature(gpgme_user_id_t uid, gpgme_key_sig_t sig)
@@ -751,10 +826,10 @@ gpgme_key_sig_t verify_signature(gpgme_user_id_t uid, gpgme_key_sig_t sig)
             }
         }
     }
-    return 0;
+    return nullptr;
 }
 
-UserID::Signature::Signature() : key(), uid(0), sig(0) {}
+UserID::Signature::Signature() : key(), uid(nullptr), sig(nullptr) {}
 
 UserID::Signature::Signature(const shared_gpgme_key_t &k, gpgme_user_id_t u, unsigned int idx)
     : key(k), uid(verify_uid(k, u)), sig(find_signature(uid, idx))
@@ -775,7 +850,7 @@ UserID UserID::Signature::parent() const
 
 const char *UserID::Signature::signerKeyID() const
 {
-    return sig ? sig->keyid : 0 ;
+    return sig ? sig->keyid : nullptr ;
 }
 
 const char *UserID::Signature::algorithmAsString() const
@@ -825,22 +900,22 @@ bool UserID::Signature::isExportable() const
 
 const char *UserID::Signature::signerUserID() const
 {
-    return sig ? sig->uid : 0 ;
+    return sig ? sig->uid : nullptr ;
 }
 
 const char *UserID::Signature::signerName() const
 {
-    return sig ? sig->name : 0 ;
+    return sig ? sig->name : nullptr ;
 }
 
 const char *UserID::Signature::signerEmail() const
 {
-    return sig ? sig->email : 0 ;
+    return sig ? sig->email : nullptr ;
 }
 
 const char *UserID::Signature::signerComment() const
 {
-    return sig ? sig->comment : 0 ;
+    return sig ? sig->comment : nullptr ;
 }
 
 unsigned int UserID::Signature::certClass() const
@@ -923,14 +998,14 @@ std::vector<Notation> UserID::Signature::notations() const
 const char *UserID::Signature::policyURL() const
 {
     if (!sig) {
-        return 0;
+        return nullptr;
     }
     for (gpgme_sig_notation_t nota = sig->notations ; nota ; nota = nota->next) {
         if (!nota->name) {
             return nota->value;
         }
     }
-    return 0;
+    return nullptr;
 }
 
 std::string UserID::addrSpecFromString(const char *userid)
@@ -1033,6 +1108,26 @@ time_t Key::lastUpdate() const
     return static_cast<time_t>(key ? key->last_update : 0);
 }
 
+bool Key::isBad() const
+{
+    return isNull() || isRevoked() || isExpired() || isDisabled() || isInvalid();
+}
+
+bool Subkey::isBad() const
+{
+    return isNull() || isRevoked() || isExpired() || isDisabled() || isInvalid();
+}
+
+bool UserID::isBad() const
+{
+    return isNull() || isRevoked() || isInvalid();
+}
+
+bool UserID::Signature::isBad() const
+{
+    return isNull() || isExpired() || isInvalid();
+}
+
 std::ostream &operator<<(std::ostream &os, const UserID &uid)
 {
     os << "GpgME::UserID(";
