@@ -14,7 +14,8 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, see <https://www.gnu.org/licenses/>.
+ * License along with this program; if not, see <https://gnu.org/licenses/>.
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 /*
@@ -96,6 +97,7 @@ struct engine_llass
     int gpg_agent:1;  /* Assume this is a gpg-agent connection.  */
   } opt;
 
+  char request_origin[10];  /* Copy from the CTX.  */
 };
 typedef struct engine_llass *engine_llass_t;
 
@@ -365,6 +367,24 @@ llass_new (void **engine, const char *file_name, const char *home_dir,
 }
 
 
+/* Copy flags from CTX into the engine object.  */
+static void
+llass_set_engine_flags (void *engine, const gpgme_ctx_t ctx)
+{
+  engine_llass_t llass = engine;
+
+  if (ctx->request_origin)
+    {
+      if (strlen (ctx->request_origin) + 1 > sizeof llass->request_origin)
+        strcpy (llass->request_origin, "xxx"); /* Too long  - force error */
+      else
+        strcpy (llass->request_origin, ctx->request_origin);
+    }
+  else
+    *llass->request_origin = 0;
+}
+
+
 static gpgme_error_t
 llass_set_locale (void *engine, int category, const char *value)
 {
@@ -473,13 +493,13 @@ llass_status_handler (void *opaque, int fd)
 	     case, we are done for now.  */
 	  if (gpg_err_code (err) == GPG_ERR_EAGAIN)
 	    {
-	      TRACE1 (DEBUG_CTX, "gpgme:llass_status_handler", llass,
+	      TRACE (DEBUG_CTX, "gpgme:llass_status_handler", llass,
 		      "fd 0x%x: EAGAIN reading assuan line (ignored)", fd);
 	      err = 0;
 	      continue;
 	    }
 
-	  TRACE2 (DEBUG_CTX, "gpgme:llass_status_handler", llass,
+	  TRACE (DEBUG_CTX, "gpgme:llass_status_handler", llass,
 		  "fd 0x%x: error reading assuan line: %s",
                   fd, gpg_strerror (err));
 	}
@@ -510,7 +530,7 @@ llass_status_handler (void *opaque, int fd)
             err = llass->user.data_cb (llass->user.data_cb_value,
                                        src, linelen);
 
-          TRACE2 (DEBUG_CTX, "gpgme:llass_status_handler", llass,
+          TRACE (DEBUG_CTX, "gpgme:llass_status_handler", llass,
 		  "fd 0x%x: D inlinedata; status from cb: %s",
                   fd, (llass->user.data_cb ?
                        (err? gpg_strerror (err):"ok"):"no callback"));
@@ -523,7 +543,7 @@ llass_status_handler (void *opaque, int fd)
           if (llass->user.data_cb)
             err = llass->user.data_cb (llass->user.data_cb_value, NULL, 0);
 
-          TRACE2 (DEBUG_CTX, "gpgme:llass_status_handler", llass,
+          TRACE (DEBUG_CTX, "gpgme:llass_status_handler", llass,
 		  "fd 0x%x: END line; status from cb: %s",
                   fd, (llass->user.data_cb ?
                        (err? gpg_strerror (err):"ok"):"no callback"));
@@ -549,7 +569,7 @@ llass_status_handler (void *opaque, int fd)
             err = llass->user.status_cb (llass->user.status_cb_value,
                                          src, args);
 
-          TRACE3 (DEBUG_CTX, "gpgme:llass_status_handler", llass,
+          TRACE (DEBUG_CTX, "gpgme:llass_status_handler", llass,
 		  "fd 0x%x: S line (%s) - status from cb: %s",
                   fd, line+2, (llass->user.status_cb ?
                                (err? gpg_strerror (err):"ok"):"no callback"));
@@ -595,7 +615,7 @@ llass_status_handler (void *opaque, int fd)
 	    err = atoi (line+4);
 	  else
 	    err = gpg_error (GPG_ERR_GENERAL);
-          TRACE2 (DEBUG_CTX, "gpgme:llass_status_handler", llass,
+          TRACE (DEBUG_CTX, "gpgme:llass_status_handler", llass,
 		  "fd 0x%x: ERR line: %s",
                   fd, err ? gpg_strerror (err) : "ok");
 
@@ -612,7 +632,7 @@ llass_status_handler (void *opaque, int fd)
 	       && line[0] == 'O' && line[1] == 'K'
 	       && (line[2] == '\0' || line[2] == ' '))
 	{
-          TRACE1 (DEBUG_CTX, "gpgme:llass_status_handler", llass,
+          TRACE (DEBUG_CTX, "gpgme:llass_status_handler", llass,
 		  "fd 0x%x: OK line", fd);
 
 	  llass->last_op_err = 0;
@@ -637,8 +657,8 @@ add_io_cb (engine_llass_t llass, iocb_data_t *iocbd, gpgme_io_cb_t handler)
 {
   gpgme_error_t err;
 
-  TRACE_BEG2 (DEBUG_ENGINE, "engine-assuan:add_io_cb", llass,
-              "fd %d, dir %d", iocbd->fd, iocbd->dir);
+  TRACE_BEG  (DEBUG_ENGINE, "engine-assuan:add_io_cb", llass,
+              "fd=%d, dir %d", iocbd->fd, iocbd->dir);
   err = (*llass->io_cbs.add) (llass->io_cbs.add_priv,
 			      iocbd->fd, iocbd->dir,
 			      handler, iocbd->data, &iocbd->tag);
@@ -660,6 +680,21 @@ start (engine_llass_t llass, const char *command)
   int nfds;
   int i;
 
+  if (*llass->request_origin && llass->opt.gpg_agent)
+    {
+      char *cmd;
+
+      cmd = _gpgme_strconcat ("OPTION pretend-request-origin=",
+                              llass->request_origin, NULL);
+      if (!cmd)
+        return gpg_error_from_syserror ();
+      err = assuan_transact (llass->assuan_ctx, cmd, NULL, NULL, NULL,
+                             NULL, NULL, NULL);
+      free (cmd);
+      if (err && gpg_err_code (err) != GPG_ERR_UNKNOWN_OPTION)
+        return err;
+    }
+
   /* We need to know the fd used by assuan for reads.  We do this by
      using the assumption that the first returned fd from
      assuan_get_active_fds() is always this one.  */
@@ -749,7 +784,7 @@ llass_io_event (void *engine, gpgme_event_io_t type, void *type_data)
 {
   engine_llass_t llass = engine;
 
-  TRACE3 (DEBUG_ENGINE, "gpgme:llass_io_event", llass,
+  TRACE (DEBUG_ENGINE, "gpgme:llass_io_event", llass,
           "event %p, type %d, type_data %p",
           llass->io_cbs.event, type, type_data);
   if (llass->io_cbs.event)
@@ -775,6 +810,7 @@ struct engine_ops _gpgme_engine_ops_assuan =
     NULL,               /* set_colon_line_handler */
     llass_set_locale,
     NULL,		/* set_protocol */
+    llass_set_engine_flags,
     NULL,               /* decrypt */
     NULL,               /* delete */
     NULL,		/* edit */
@@ -790,7 +826,6 @@ struct engine_ops _gpgme_engine_ops_assuan =
     NULL,               /* keysign */
     NULL,               /* tofu_policy */
     NULL,               /* sign */
-    NULL,		/* trustlist */
     NULL,               /* verify */
     NULL,               /* getauditlog */
     llass_transact,     /* opassuan_transact */
