@@ -1,23 +1,24 @@
 /* engine-gpg.c - Gpg Engine.
-   Copyright (C) 2000 Werner Koch (dd9jn)
-   Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007,
-                 2009, 2010, 2012, 2013 g10 Code GmbH
-
-   This file is part of GPGME.
-
-   GPGME is free software; you can redistribute it and/or modify it
-   under the terms of the GNU Lesser General Public License as
-   published by the Free Software Foundation; either version 2.1 of
-   the License, or (at your option) any later version.
-
-   GPGME is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Lesser General Public License for more details.
-
-   You should have received a copy of the GNU Lesser General Public
-   License along with this program; if not, see <https://www.gnu.org/licenses/>.
-*/
+ * Copyright (C) 2000 Werner Koch (dd9jn)
+ * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007,
+ *               2009, 2010, 2012, 2013 g10 Code GmbH
+ *
+ * This file is part of GPGME.
+ *
+ * GPGME is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2.1 of
+ * the License, or (at your option) any later version.
+ *
+ * GPGME is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this program; if not, see <https://gnu.org/licenses/>.
+ * SPDX-License-Identifier: LGPL-2.1-or-later
+ */
 
 #if HAVE_CONFIG_H
 #include <config.h>
@@ -26,7 +27,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include <assert.h>
-#include <errno.h>
 #ifdef HAVE_UNISTD_H
 # include <unistd.h>
 #endif
@@ -43,6 +43,7 @@
 #include "sema.h"
 #include "debug.h"
 #include "data.h"
+#include "mbox-util.h"
 
 #include "engine-backend.h"
 
@@ -135,17 +136,27 @@ struct engine_gpg
     char *keyword;       /* what has been requested (malloced) */
     engine_command_handler_t fnc;
     void *fnc_value;
-    /* The kludges never end.  This is used to couple command handlers
-       with output data in edit key mode.  */
-    gpgme_data_t linked_data;
-    int linked_idx;
   } cmd;
 
   struct gpgme_io_cbs io_cbs;
   gpgme_pinentry_mode_t pinentry_mode;
+  char request_origin[10];
+  char *auto_key_locate;
+  char *trust_model;
+
+  struct {
+    unsigned int no_symkey_cache : 1;
+    unsigned int offline : 1;
+    unsigned int ignore_mdc_error : 1;
+    unsigned int include_key_block : 1;
+    unsigned int auto_key_import : 1;
+  } flags;
 
   /* NULL or the data object fed to --override_session_key-fd.  */
   gpgme_data_t override_session_key;
+
+  /* Memory data containing diagnostics (--logger-fd) of gpg */
+  gpgme_data_t diagnostics;
 };
 
 typedef struct engine_gpg *engine_gpg_t;
@@ -156,7 +167,7 @@ gpg_io_event (void *engine, gpgme_event_io_t type, void *type_data)
 {
   engine_gpg_t gpg = engine;
 
-  TRACE3 (DEBUG_ENGINE, "gpgme:gpg_io_event", gpg,
+  TRACE (DEBUG_ENGINE, "gpgme:gpg_io_event", gpg,
           "event %p, type %d, type_data %p",
           gpg->io_cbs.event, type, type_data);
   if (gpg->io_cbs.event)
@@ -447,8 +458,11 @@ gpg_release (void *engine)
     free_argv (gpg->argv);
   if (gpg->cmd.keyword)
     free (gpg->cmd.keyword);
+  free (gpg->auto_key_locate);
+  free (gpg->trust_model);
 
   gpgme_data_release (gpg->override_session_key);
+  gpgme_data_release (gpg->diagnostics);
 
   free (gpg);
 }
@@ -496,8 +510,6 @@ gpg_new (void **engine, const char *file_name, const char *home_dir,
   gpg->colon.fd[1] = -1;
   gpg->cmd.fd = -1;
   gpg->cmd.idx = -1;
-  gpg->cmd.linked_data = NULL;
-  gpg->cmd.linked_idx = -1;
 
   /* Allocate the read buffer for the status pipe.  */
   gpg->status.bufsize = 1024;
@@ -619,6 +631,16 @@ gpg_new (void **engine, const char *file_name, const char *home_dir,
 	}
     }
 
+  rc = gpgme_data_new (&gpg->diagnostics);
+  if (rc)
+    goto leave;
+
+  rc = add_arg (gpg, "--logger-fd");
+  if (rc)
+    goto leave;
+
+  rc = add_data (gpg, gpg->diagnostics, -2, 1);
+
  leave:
   if (rc)
     gpg_release (gpg);
@@ -628,6 +650,54 @@ gpg_new (void **engine, const char *file_name, const char *home_dir,
 }
 
 
+/* Copy flags from CTX into the engine object.  */
+static void
+gpg_set_engine_flags (void *engine, const gpgme_ctx_t ctx)
+{
+  engine_gpg_t gpg = engine;
+
+  if (ctx->request_origin && have_gpg_version (gpg, "2.2.6"))
+    {
+      if (strlen (ctx->request_origin) + 1 > sizeof gpg->request_origin)
+        strcpy (gpg->request_origin, "xxx"); /* Too long  - force error */
+      else
+        strcpy (gpg->request_origin, ctx->request_origin);
+    }
+  else
+    *gpg->request_origin = 0;
+
+  if (ctx->auto_key_locate && have_gpg_version (gpg, "2.1.18"))
+    {
+      if (gpg->auto_key_locate)
+        free (gpg->auto_key_locate);
+      gpg->auto_key_locate = _gpgme_strconcat ("--auto-key-locate=",
+                                               ctx->auto_key_locate, NULL);
+    }
+
+  if (ctx->trust_model && strlen (ctx->trust_model))
+    {
+      if (gpg->trust_model)
+        free (gpg->trust_model);
+      gpg->trust_model = _gpgme_strconcat ("--trust-model=",
+                                           ctx->trust_model, NULL);
+    }
+
+  gpg->flags.no_symkey_cache = (ctx->no_symkey_cache
+                                && have_gpg_version (gpg, "2.2.7"));
+  gpg->flags.offline = (ctx->offline && have_gpg_version (gpg, "2.1.23"));
+
+  gpg->flags.ignore_mdc_error = !!ctx->ignore_mdc_error;
+
+  if (have_gpg_version (gpg, "2.2.20"))
+    {
+      if (ctx->auto_key_import)
+        gpg->flags.auto_key_import = 1;
+      if (ctx->include_key_block)
+        gpg->flags.include_key_block = 1;
+    }
+}
+
+
 static gpgme_error_t
 gpg_set_locale (void *engine, int category, const char *value)
 {
@@ -685,7 +755,7 @@ gpg_set_status_cb (void *engine, gpgme_status_cb_t cb, void *cb_value)
 }
 
 
-/* Note, that the status_handler is allowed to modifiy the args
+/* Note, that the status_handler is allowed to modify the args
    value.  */
 static void
 gpg_set_status_handler (void *engine, engine_status_handler_t fnc,
@@ -762,14 +832,14 @@ command_handler (void *opaque, int fd)
 
 
 
-/* The Fnc will be called to get a value for one of the commands with
-   a key KEY.  If the Code passed to FNC is 0, the function may release
-   resources associated with the returned value from another call.  To
-   match such a second call to a first call, the returned value from
-   the first call is passed as keyword.  */
+/* The FNC will be called to get a value for one of the commands with
+ * a key KEY.  If the code passed to FNC is 0, the function may
+ * release resources associated with the returned value from another
+ * call.  To match such a second call to a first call, the returned
+ * value from the first call is passed as keyword.  */
 static gpgme_error_t
 gpg_set_command_handler (void *engine, engine_command_handler_t fnc,
-			 void *fnc_value, gpgme_data_t linked_data)
+			 void *fnc_value)
 {
   engine_gpg_t gpg = engine;
   gpgme_error_t rc;
@@ -788,7 +858,6 @@ gpg_set_command_handler (void *engine, engine_command_handler_t fnc,
   gpg->cmd.fnc = fnc;
   gpg->cmd.cb_data = (void *) &gpg->cmd;
   gpg->cmd.fnc_value = fnc_value;
-  gpg->cmd.linked_data = linked_data;
   gpg->cmd.used = 1;
   return 0;
 }
@@ -800,7 +869,7 @@ build_argv (engine_gpg_t gpg, const char *pgmname)
   gpgme_error_t err;
   struct arg_and_data_s *a;
   struct fd_data_map_s *fd_data_map;
-  size_t datac=0, argc=0;
+  size_t datac=0, argc=0, allocated_argc=0;
   char **argv;
   int need_special = 0;
   int use_agent = 0;
@@ -848,17 +917,33 @@ build_argv (engine_gpg_t gpg, const char *pgmname)
 	  /*   fprintf (stderr, "build_argv: arg=`%s'\n", a->arg );*/
         }
     }
+
   if (need_special)
     argc++;
   if (use_agent)
     argc++;
+  if (*gpg->request_origin)
+    argc++;
+  if (gpg->auto_key_locate)
+    argc++;
+  if (gpg->trust_model)
+    argc++;
+  if (gpg->flags.no_symkey_cache)
+    argc++;
+  if (gpg->flags.ignore_mdc_error)
+    argc++;
+  if (gpg->flags.offline)
+    argc++;
   if (gpg->pinentry_mode)
     argc++;
   if (!gpg->cmd.used)
-    argc++;	/* --batch */
-  argc += 1;	/* --no-sk-comments */
+    argc++; /* --batch */
+
+  argc++;   /* --no-sk-comments */
 
   argv = calloc (argc + 1, sizeof *argv);
+  allocated_argc = argc;
+
   if (!argv)
     return gpg_error_from_syserror ();
   fd_data_map = calloc (datac + 1, sizeof *fd_data_map);
@@ -903,6 +988,87 @@ build_argv (engine_gpg_t gpg, const char *pgmname)
         }
       argc++;
     }
+  /* NOTE: If you add a new argument here. Ensure that
+     argc is counted up above to allocate enough memory. */
+
+  if (*gpg->request_origin)
+    {
+      argv[argc] = _gpgme_strconcat ("--request-origin=",
+                                     gpg->request_origin, NULL);
+      if (!argv[argc])
+	{
+          int saved_err = gpg_error_from_syserror ();
+	  free (fd_data_map);
+	  free_argv (argv);
+	  return saved_err;
+        }
+      argc++;
+    }
+
+  if (gpg->auto_key_locate)
+    {
+      argv[argc] = strdup (gpg->auto_key_locate);
+      if (!argv[argc])
+        {
+          int saved_err = gpg_error_from_syserror ();
+          free (fd_data_map);
+          free_argv (argv);
+          return saved_err;
+        }
+      argc++;
+    }
+
+  if (gpg->trust_model)
+    {
+      argv[argc] = strdup (gpg->trust_model);
+      if (!argv[argc])
+        {
+          int saved_err = gpg_error_from_syserror ();
+          free (fd_data_map);
+          free_argv (argv);
+          return saved_err;
+        }
+      argc++;
+    }
+
+  if (gpg->flags.no_symkey_cache)
+    {
+      argv[argc] = strdup ("--no-symkey-cache");
+      if (!argv[argc])
+	{
+          int saved_err = gpg_error_from_syserror ();
+	  free (fd_data_map);
+	  free_argv (argv);
+	  return saved_err;
+        }
+      argc++;
+    }
+
+  if (gpg->flags.ignore_mdc_error)
+    {
+      argv[argc] = strdup ("--ignore-mdc-error");
+      if (!argv[argc])
+	{
+          int saved_err = gpg_error_from_syserror ();
+	  free (fd_data_map);
+	  free_argv (argv);
+	  return saved_err;
+        }
+      argc++;
+    }
+
+  if (gpg->flags.offline)
+    {
+      argv[argc] = strdup ("--disable-dirmngr");
+      if (!argv[argc])
+	{
+          int saved_err = gpg_error_from_syserror ();
+	  free (fd_data_map);
+	  free_argv (argv);
+	  return saved_err;
+        }
+      argc++;
+    }
 
   if (gpg->pinentry_mode && have_gpg_version (gpg, "2.1.0"))
     {
@@ -967,10 +1133,10 @@ build_argv (engine_gpg_t gpg, const char *pgmname)
 	    if (_gpgme_io_pipe (fds, fd_data_map[datac].inbound ? 1 : 0)
 		== -1)
 	      {
-		int saved_errno = errno;
+		int saved_err = gpg_error_from_syserror ();
 		free (fd_data_map);
 		free_argv (argv);
-		return gpg_error (saved_errno);
+		return saved_err;
 	      }
 	    if (_gpgme_io_set_close_notify (fds[0],
 					    close_notify_handler, gpg)
@@ -1005,11 +1171,6 @@ build_argv (engine_gpg_t gpg, const char *pgmname)
 		  assert (gpg->cmd.idx == -1);
 		  gpg->cmd.idx = datac;
 		}
-	      else if (gpg->cmd.linked_data == a->data)
-		{
-		  assert (gpg->cmd.linked_idx == -1);
-		  gpg->cmd.linked_idx = datac;
-		}
 	    }
 
 	  fd_data_map[datac].data = a->data;
@@ -1056,6 +1217,11 @@ build_argv (engine_gpg_t gpg, const char *pgmname)
             argc++;
         }
     }
+  /* Saveguard against adding a new argument without properly
+     counting up the argc used for allocation at the beginning
+     of this function. It would be better to use a dynamically
+     allocated array like ccparray in gnupg. */
+  assert (argc <= allocated_argc);
 
   gpg->argv = argv;
   gpg->fd_data_map = fd_data_map;
@@ -1178,7 +1344,7 @@ read_status (engine_gpg_t gpg)
 			     received and the next thing will be that
 			     the command handler does its action.  */
 			  if (nread > 1)
-			    TRACE0 (DEBUG_CTX, "gpgme:read_status", 0,
+			    TRACE (DEBUG_CTX, "gpgme:read_status", 0,
 				    "error: unexpected data");
 
 			  add_io_cb (gpg, gpg->cmd.fd, 0,
@@ -1196,44 +1362,6 @@ read_status (engine_gpg_t gpg)
 			  if (err)
 			    return err;
                         }
-
-		      if (r == GPGME_STATUS_END_STREAM)
-			{
-			  if (gpg->cmd.used)
-			    {
-			      /* Before we can actually add the
-				 command fd, we might have to flush
-				 the linked output data pipe.  */
-			      if (gpg->cmd.linked_idx != -1
-				  && gpg->fd_data_map[gpg->cmd.linked_idx].fd
-				  != -1)
-				{
-				  struct io_select_fd_s fds;
-				  fds.fd =
-				    gpg->fd_data_map[gpg->cmd.linked_idx].fd;
-				  fds.for_read = 1;
-				  fds.for_write = 0;
-				  fds.opaque = NULL;
-				  do
-				    {
-				      fds.signaled = 0;
-				      _gpgme_io_select (&fds, 1, 1);
-				      if (fds.signaled)
-					_gpgme_data_inbound_handler
-					  (gpg->cmd.linked_data, fds.fd);
-				    }
-				  while (fds.signaled);
-				}
-
-			      /* XXX We must check if there are any
-				 more fds active after removing this
-				 one.  */
-			      (*gpg->io_cbs.remove)
-				(gpg->fd_data_map[gpg->cmd.idx].tag);
-			      gpg->cmd.fd = gpg->fd_data_map[gpg->cmd.idx].fd;
-			      gpg->fd_data_map[gpg->cmd.idx].fd = -1;
-			    }
-                        }
                     }
                 }
 	      /* To reuse the buffer for the next line we have to
@@ -1584,12 +1712,15 @@ gpg_decrypt (void *engine,
   if (!err && auto_key_retrieve)
     err = add_arg (gpg, "--auto-key-retrieve");
 
+  if (!err && gpg->flags.auto_key_import)
+    err = add_arg (gpg, "--auto-key-import");
+
   if (!err && override_session_key && *override_session_key)
     {
       if (have_gpg_version (gpg, "2.1.16"))
         {
           gpgme_data_release (gpg->override_session_key);
-          TRACE2 (DEBUG_ENGINE, "override", gpg, "seskey='%s' len=%zu\n",
+          TRACE (DEBUG_ENGINE, "override", gpg, "seskey='%s' len=%zu\n",
                   override_session_key,
                   strlen (override_session_key));
 
@@ -1598,7 +1729,17 @@ gpg_decrypt (void *engine,
                                          strlen (override_session_key), 1);
           if (!err)
             {
-              err = add_arg (gpg, "--override-session-key-fd");
+              /* When we are not trying to verify signatures as well,
+               * we add --no-keyring because a keyring is not required
+               * for decryption when overriding the session key.  It would
+               * work without that option but --no-keyring avoids that
+               * gpg return a failure due to a missing key log_error()
+               * diagnostic.  --no-keyring is supported since 2.1.14. */
+
+              if (!(flags & GPGME_DECRYPT_VERIFY))
+                  err = add_arg (gpg, "--no-keyring");
+              if (!err)
+                err = add_arg (gpg, "--override-session-key-fd");
               if (!err)
                 err = add_data (gpg, gpg->override_session_key, -2, 0);
             }
@@ -1722,8 +1863,13 @@ append_args_from_sender (engine_gpg_t gpg, gpgme_ctx_t ctx)
 }
 
 
+#define NOTATION_FLAG_SIG  1 /* Use --sig-notation (default)*/
+#define NOTATION_FLAG_CERT 2 /* Use --cert-notation */
+#define NOTATION_FLAG_SET  3 /* Use --set-notation */
+
 static gpgme_error_t
-append_args_from_sig_notations (engine_gpg_t gpg, gpgme_ctx_t ctx /* FIXME */)
+append_args_from_sig_notations (engine_gpg_t gpg, gpgme_ctx_t ctx /* FIXME */,
+                                int flags)
 {
   gpgme_error_t err = 0;
   gpgme_sig_notation_t notation;
@@ -1764,7 +1910,14 @@ append_args_from_sig_notations (engine_gpg_t gpg, gpgme_ctx_t ctx /* FIXME */)
 	    }
 
 	  if (!err)
-	    err = add_arg (gpg, "--sig-notation");
+            {
+              if ((flags & NOTATION_FLAG_SET))
+                err = add_arg (gpg, "--set-notation");
+              else if ((flags & NOTATION_FLAG_CERT))
+                err = add_arg (gpg, "--cert-notation");
+              else
+                err = add_arg (gpg, "--sig-notation");
+            }
 	  if (!err)
 	    err = add_arg (gpg, arg);
 
@@ -1815,10 +1968,15 @@ gpg_edit (void *engine, int type, gpgme_key_t key, gpgme_data_t out,
   gpgme_error_t err;
 
   err = add_arg (gpg, "--with-colons");
+
+  if (!err && ctx->extended_edit)
+    err = add_arg (gpg, "--expert");
   if (!err)
     err = append_args_from_signers (gpg, ctx);
   if (!err)
-  err = add_arg (gpg, type == 0 ? "--edit-key" : "--card-edit");
+    err = append_args_from_sig_notations (gpg, ctx, NOTATION_FLAG_CERT);
+  if (!err)
+    err = add_arg (gpg, type == 0 ? "--edit-key" : "--card-edit");
   if (!err)
     err = add_data (gpg, out, 1, 1);
   if (!err)
@@ -1838,8 +1996,70 @@ gpg_edit (void *engine, int type, gpgme_key_t key, gpgme_data_t out,
 }
 
 
+/* Add a single argument from a key to an -r option.  */
+static gpg_error_t
+add_arg_recipient (engine_gpg_t gpg, gpgme_encrypt_flags_t flags,
+                   gpgme_key_t key)
+{
+  gpg_error_t err;
+
+  if ((flags & GPGME_ENCRYPT_WANT_ADDRESS))
+    {
+      /* We have no way to figure out which mail address was
+       * requested.  FIXME: It would be possible to figure this out by
+       * consulting the SENDER property of the context.  */
+      err = gpg_error (GPG_ERR_INV_USER_ID);
+    }
+  else
+    err = add_arg (gpg, key->subkeys->fpr);
+
+  return err;
+}
+
+
+/* Add a single argument from a USERID string to an -r option.  */
+static gpg_error_t
+add_arg_recipient_string (engine_gpg_t gpg, gpgme_encrypt_flags_t flags,
+                          const char *userid, int useridlen)
+{
+  gpg_error_t err;
+
+  if ((flags & GPGME_ENCRYPT_WANT_ADDRESS))
+    {
+      char *tmpstr, *mbox;
+
+      tmpstr = malloc (useridlen + 1);
+      if (!tmpstr)
+        err = gpg_error_from_syserror ();
+      else
+        {
+          memcpy (tmpstr, userid, useridlen);
+          tmpstr[useridlen] = 0;
+
+          mbox = _gpgme_mailbox_from_userid (tmpstr);
+          if (!mbox)
+            {
+              err = gpg_error_from_syserror ();
+              if (gpg_err_code (err) == GPG_ERR_EINVAL)
+                err = gpg_error (GPG_ERR_INV_USER_ID);
+            }
+          else
+            err = add_arg (gpg, mbox);
+
+          free (mbox);
+          free (tmpstr);
+        }
+    }
+  else
+    err = add_arg_len (gpg, NULL, userid, useridlen);
+
+  return err;
+}
+
+
 static gpgme_error_t
-append_args_from_recipients (engine_gpg_t gpg, gpgme_key_t recp[])
+append_args_from_recipients (engine_gpg_t gpg, gpgme_encrypt_flags_t flags,
+                             gpgme_key_t recp[])
 {
   gpgme_error_t err = 0;
   int i = 0;
@@ -1851,7 +2071,7 @@ append_args_from_recipients (engine_gpg_t gpg, gpgme_key_t recp[])
       if (!err)
 	err = add_arg (gpg, "-r");
       if (!err)
-	err = add_arg (gpg, recp[i]->subkeys->fpr);
+	err = add_arg_recipient (gpg, flags, recp[i]);
       if (err)
 	break;
       i++;
@@ -1860,17 +2080,88 @@ append_args_from_recipients (engine_gpg_t gpg, gpgme_key_t recp[])
 }
 
 
+/* Take recipients from the LF delimited STRING and add -r args.  */
+static gpg_error_t
+append_args_from_recipients_string (engine_gpg_t gpg,
+                                    gpgme_encrypt_flags_t flags,
+                                    const char *string)
+{
+  gpg_error_t err = 0;
+  gpgme_encrypt_flags_t orig_flags = flags;
+  int any = 0;
+  int ignore = 0;
+  int hidden = 0;
+  int file = 0;
+  const char *s;
+  int n;
+
+  do
+    {
+      /* Skip leading white space */
+      while (*string == ' ' || *string == '\t')
+        string++;
+      if (!*string)
+        break;
+
+      /* Look for the LF. */
+      s = strchr (string, '\n');
+      if (s)
+        n = s - string;
+      else
+        n = strlen (string);
+      while (n && (string[n-1] == ' ' || string[n-1] == '\t'))
+        n--;
+
+      if (!ignore && n == 2 && !memcmp (string, "--", 2))
+        ignore = 1;
+      else if (!ignore && n == 8 && !memcmp (string, "--hidden", 8))
+        hidden = 1;
+      else if (!ignore && n == 11 && !memcmp (string, "--no-hidden", 11))
+        hidden = 0;
+      else if (!ignore && n == 6 && !memcmp (string, "--file", 6))
+        {
+          file = 1;
+          /* Because the key is used as is we need to ignore this flag:  */
+          flags &= ~GPGME_ENCRYPT_WANT_ADDRESS;
+        }
+      else if (!ignore && n == 9 && !memcmp (string, "--no-file", 9))
+        {
+          file = 0;
+          flags = orig_flags;
+        }
+      else if (!ignore && n > 2 && !memcmp (string, "--", 2))
+        err = gpg_error (GPG_ERR_UNKNOWN_OPTION);
+      else if (n) /* Not empty - use it.  */
+        {
+          err = add_arg (gpg, file? (hidden? "-F":"-f") : (hidden? "-R":"-r"));
+          if (!err)
+            err = add_arg_recipient_string (gpg, flags, string, n);
+          if (!err)
+            any = 1;
+        }
+
+      string += n + !!s;
+    }
+  while (!err);
+
+  if (!err && !any)
+    err = gpg_error (GPG_ERR_MISSING_KEY);
+  return err;
+}
+
+
 static gpgme_error_t
-gpg_encrypt (void *engine, gpgme_key_t recp[], gpgme_encrypt_flags_t flags,
+gpg_encrypt (void *engine, gpgme_key_t recp[], const char *recpstring,
+             gpgme_encrypt_flags_t flags,
 	     gpgme_data_t plain, gpgme_data_t ciph, int use_armor)
 {
   engine_gpg_t gpg = engine;
   gpgme_error_t err = 0;
 
-  if (recp)
+  if (recp || recpstring)
     err = add_arg (gpg, "--encrypt");
 
-  if (!err && ((flags & GPGME_ENCRYPT_SYMMETRIC) || !recp))
+  if (!err && ((flags & GPGME_ENCRYPT_SYMMETRIC) || (!recp && !recpstring)))
     err = add_arg (gpg, "--symmetric");
 
   if (!err && use_armor)
@@ -1897,7 +2188,10 @@ gpg_encrypt (void *engine, gpgme_key_t recp[], gpgme_encrypt_flags_t flags,
       && have_gpg_version (gpg, "2.1.14"))
     err = add_arg (gpg, "--mimemode");
 
-  if (recp)
+  if (!err && gpg->flags.include_key_block)
+    err = add_arg (gpg, "--include-key-block");
+
+  if (recp || recpstring)
     {
       /* If we know that all recipients are valid (full or ultimate trust)
 	 we can suppress further checks.  */
@@ -1907,8 +2201,10 @@ gpg_encrypt (void *engine, gpgme_key_t recp[], gpgme_encrypt_flags_t flags,
       if (!err && (flags & GPGME_ENCRYPT_NO_ENCRYPT_TO))
 	err = add_arg (gpg, "--no-encrypt-to");
 
-      if (!err)
-	err = append_args_from_recipients (gpg, recp);
+      if (!err && !recp && recpstring)
+	err = append_args_from_recipients_string (gpg, flags, recpstring);
+      else if (!err)
+	err = append_args_from_recipients (gpg, flags, recp);
     }
 
   /* Tell the gpg object about the data.  */
@@ -1941,6 +2237,7 @@ gpg_encrypt (void *engine, gpgme_key_t recp[], gpgme_encrypt_flags_t flags,
 
 static gpgme_error_t
 gpg_encrypt_sign (void *engine, gpgme_key_t recp[],
+                  const char *recpstring,
 		  gpgme_encrypt_flags_t flags, gpgme_data_t plain,
 		  gpgme_data_t ciph, int use_armor,
 		  gpgme_ctx_t ctx /* FIXME */)
@@ -1948,10 +2245,10 @@ gpg_encrypt_sign (void *engine, gpgme_key_t recp[],
   engine_gpg_t gpg = engine;
   gpgme_error_t err = 0;
 
-  if (recp)
+  if (recp || recpstring)
     err = add_arg (gpg, "--encrypt");
 
-  if (!err && ((flags & GPGME_ENCRYPT_SYMMETRIC) || !recp))
+  if (!err && ((flags & GPGME_ENCRYPT_SYMMETRIC) || (!recp && !recpstring)))
     err = add_arg (gpg, "--symmetric");
 
   if (!err)
@@ -1969,7 +2266,10 @@ gpg_encrypt_sign (void *engine, gpgme_key_t recp[],
       && have_gpg_version (gpg, "2.1.14"))
     err = add_arg (gpg, "--mimemode");
 
-  if (recp)
+  if (!err && gpg->flags.include_key_block)
+    err = add_arg (gpg, "--include-key-block");
+
+  if (recp || recpstring)
     {
       /* If we know that all recipients are valid (full or ultimate trust)
 	 we can suppress further checks.  */
@@ -1979,8 +2279,10 @@ gpg_encrypt_sign (void *engine, gpgme_key_t recp[],
       if (!err && (flags & GPGME_ENCRYPT_NO_ENCRYPT_TO))
 	err = add_arg (gpg, "--no-encrypt-to");
 
-      if (!err)
-	err = append_args_from_recipients (gpg, recp);
+      if (!err && !recp && recpstring)
+	err = append_args_from_recipients_string (gpg, flags, recpstring);
+      else if (!err)
+	err = append_args_from_recipients (gpg, flags, recp);
     }
 
   if (!err)
@@ -1990,7 +2292,7 @@ gpg_encrypt_sign (void *engine, gpgme_key_t recp[],
     err = append_args_from_sender (gpg, ctx);
 
   if (!err)
-    err = append_args_from_sig_notations (gpg, ctx);
+    err = append_args_from_sig_notations (gpg, ctx, NOTATION_FLAG_SIG);
 
   /* Tell the gpg object about the data.  */
   if (!err)
@@ -2028,17 +2330,36 @@ export_common (engine_gpg_t gpg, gpgme_export_mode_t mode,
 
   if ((mode & ~(GPGME_EXPORT_MODE_EXTERN
                 |GPGME_EXPORT_MODE_MINIMAL
+                |GPGME_EXPORT_MODE_SSH
                 |GPGME_EXPORT_MODE_SECRET)))
     return gpg_error (GPG_ERR_NOT_SUPPORTED);
 
   if ((mode & GPGME_EXPORT_MODE_MINIMAL))
-    err = add_arg (gpg, "--export-options=export-minimal");
+    {
+      if ((mode & GPGME_EXPORT_MODE_NOUID))
+        err = add_arg (gpg, "--export-options=export-minimal,export-drop-uids");
+      else
+        err = add_arg (gpg, "--export-options=export-minimal");
+    }
+  else if ((mode & GPGME_EXPORT_MODE_NOUID))
+    err = add_arg (gpg, "--export-options=export-drop-uids");
 
   if (err)
     ;
+  else if ((mode & GPGME_EXPORT_MODE_SSH))
+    {
+      if (have_gpg_version (gpg, "2.1.11"))
+        err = add_arg (gpg, "--export-ssh-key");
+      else
+        err = gpg_error (GPG_ERR_NOT_SUPPORTED);
+      if (!err)
+        err = add_data (gpg, keydata, 1, 1);
+    }
   else if ((mode & GPGME_EXPORT_MODE_EXTERN))
     {
       err = add_arg (gpg, "--send-keys");
+      if (!err && (mode & GPGME_EXPORT_MODE_NOUID))
+        err = add_arg (gpg, "--keyserver-options=export-drop-uids");
     }
   else
     {
@@ -2686,7 +3007,15 @@ gpg_keylist_build_options (engine_gpg_t gpg, int secret_only,
     err = add_arg (gpg, "--with-tofu-info");
 
   if (!err && (mode & GPGME_KEYLIST_MODE_WITH_SECRET))
-    err = add_arg (gpg, "--with-secret");
+    {
+      err = add_arg (gpg, "--with-secret");
+      err = add_arg (gpg, "--with-keygrip");
+    }
+  else if (!err && (mode & GPGME_KEYLIST_MODE_WITH_KEYGRIP))
+    {
+      /* Explicitly requests the keygrip.  */
+      err = add_arg (gpg, "--with-keygrip");
+    }
 
   if (!err
       && (mode & GPGME_KEYLIST_MODE_SIGS)
@@ -2950,12 +3279,14 @@ gpg_sign (void *engine, gpgme_data_t in, gpgme_data_t out,
         }
     }
 
+  if (!err && gpg->flags.include_key_block)
+    err = add_arg (gpg, "--include-key-block");
   if (!err)
     err = append_args_from_signers (gpg, ctx);
   if (!err)
     err = append_args_from_sender (gpg, ctx);
   if (!err)
-    err = append_args_from_sig_notations (gpg, ctx);
+    err = append_args_from_sig_notations (gpg, ctx, NOTATION_FLAG_SIG);
 
   if (gpgme_data_get_file_name (in))
     {
@@ -2981,29 +3312,6 @@ gpg_sign (void *engine, gpgme_data_t in, gpgme_data_t out,
   return err;
 }
 
-static gpgme_error_t
-gpg_trustlist (void *engine, const char *pattern)
-{
-  engine_gpg_t gpg = engine;
-  gpgme_error_t err;
-
-  err = add_arg (gpg, "--with-colons");
-  if (!err)
-    err = add_arg (gpg, "--list-trust-path");
-
-  /* Tell the gpg object about the data.  */
-  if (!err)
-    err = add_arg (gpg, "--");
-  if (!err)
-    err = add_arg (gpg, pattern);
-
-  if (!err)
-    err = start (gpg);
-
-  return err;
-}
-
-
 static gpgme_error_t
 gpg_verify (void *engine, gpgme_data_t sig, gpgme_data_t signed_text,
 	    gpgme_data_t plaintext, gpgme_ctx_t ctx)
@@ -3012,6 +3320,8 @@ gpg_verify (void *engine, gpgme_data_t sig, gpgme_data_t signed_text,
   gpgme_error_t err;
 
   err = append_args_from_sender (gpg, ctx);
+  if (!err && gpg->flags.auto_key_import)
+    err = add_arg (gpg, "--auto-key-import");
   if (!err && ctx->auto_key_retrieve)
     err = add_arg (gpg, "--auto-key-retrieve");
 
@@ -3071,6 +3381,52 @@ gpg_set_pinentry_mode (void *engine, gpgme_pinentry_mode_t mode)
 }
 
 
+static gpgme_error_t
+gpg_getauditlog (void *engine, gpgme_data_t output, unsigned int flags)
+{
+  engine_gpg_t gpg = engine;
+#define MYBUFLEN 4096
+  char buf[MYBUFLEN];
+  int nread;
+  int any_written = 0;
+
+  if (!(flags & GPGME_AUDITLOG_DIAG))
+    {
+      return gpg_error (GPG_ERR_NOT_IMPLEMENTED);
+    }
+
+  if (!gpg || !output)
+    {
+      return gpg_error (GPG_ERR_INV_VALUE);
+    }
+
+  if (!gpg->diagnostics)
+    {
+      return gpg_error (GPG_ERR_GENERAL);
+    }
+
+  gpgme_data_rewind (gpg->diagnostics);
+
+  while ((nread = gpgme_data_read (gpg->diagnostics, buf, MYBUFLEN)) > 0)
+    {
+      any_written = 1;
+      if (gpgme_data_write (output, buf, nread) == -1)
+        return gpg_error_from_syserror ();
+    }
+  if (!any_written)
+    {
+      return gpg_error (GPG_ERR_NO_DATA);
+    }
+
+  if (nread == -1)
+    return gpg_error_from_syserror ();
+
+  gpgme_data_rewind (output);
+  return 0;
+#undef MYBUFLEN
+}
+
+
 
 struct engine_ops _gpgme_engine_ops_gpg =
   {
@@ -3090,6 +3446,7 @@ struct engine_ops _gpgme_engine_ops_gpg =
     gpg_set_colon_line_handler,
     gpg_set_locale,
     NULL,				/* set_protocol */
+    gpg_set_engine_flags,               /* set_engine_flags */
     gpg_decrypt,
     gpg_delete,
     gpg_edit,
@@ -3105,9 +3462,8 @@ struct engine_ops _gpgme_engine_ops_gpg =
     gpg_keysign,
     gpg_tofu_policy,    /* tofu_policy */
     gpg_sign,
-    gpg_trustlist,
     gpg_verify,
-    NULL,		/* getauditlog */
+    gpg_getauditlog,
     NULL,               /* opassuan_transact */
     NULL,		/* conf_load */
     NULL,		/* conf_save */
