@@ -621,8 +621,9 @@ keylist_colon_handler (void *priv, char *line)
 
   /* Only look at signature and trust info records immediately
      following a user ID.  For this, clear the user ID pointer when
-     encountering anything but a signature or trust record.  */
-  if (rectype != RT_SIG && rectype != RT_REV && rectype != RT_TFS)
+     encountering anything but a signature, trust record or subpacket.  */
+  if (rectype != RT_SIG && rectype != RT_REV && rectype != RT_TFS &&
+      rectype != RT_SPK)
     opd->tmp_uid = NULL;
 
   /* Only look at subpackets immediately following a signature.  For
@@ -830,6 +831,12 @@ keylist_colon_handler (void *priv, char *line)
 
           if (field[1])
             set_userid_flags (key, field[1]);
+          if (field[7] && *field[7])
+            {
+              gpgme_user_id_t uid = key->_last_uid;
+              assert (uid);
+              uid->uidhash = strdup (field[7]);
+            }
           opd->tmp_uid = key->_last_uid;
           if (fields >= 20)
             {
